@@ -4,29 +4,22 @@ use strict;
 use warnings;
 use Carp qw(confess);
 use Encode qw(decode find_encoding);
-use List::MoreUtils qw(uniq);
 use Locale::PO;
 use Locale::TextDomain::OO::Util::ExtractHeader;
 use Moo;
 use MooX::StrictConstructor;
 use MooX::Types::MooseLike::Base qw(CodeRef Str);
-use MooX::Types::MooseLike::Numeric qw(PositiveInt PositiveOrZeroInt);
 use Try::Tiny;
 use namespace::autoclean;
 
-our $VERSION = '0.003';
-
-# a .. w, z     => A-WZ
-# A .. W, Z     => Ym
-# space         => YX
-# open, e.g. {  => XX
-# :             => XY
-# close, e.g. } => XZ
-# other         => XAA .. XPP
-#                  like hex but
-#                  0123456789ABCDEF is
-#                  ABCDEFGHIJKLMNOP
+our $VERSION = '1.000';
 
+# plural_ref e.g. ru
+# The key is the plural form 0, 1 or 2.
+# The value is the first number 0 .. that is resulting in that plural form.
+# 0 => 1, # singular
+# 1 => 2, # 2 .. 4 plural
+# 2 => 5, # 5 .. plural
 has _plural_ref => (
     is       => 'ro',
     init_arg => undef,
@@ -41,6 +34,26 @@ sub _clear_plural_ref {
     return;
 }
 
+# to store the original gettext parts by placeholder number
+# e.g.
+# %*(%1,singular,plural,zero)
+# 1 => [ '*', 'singular', 'plural'. 'zero' ],
+# %quant(%2,singular,plural)
+# 2 => [ 'quant', 'singular', 'plural' ],
+has _gettext_ref => (
+    is       => 'ro',
+    init_arg => undef,
+    default  => sub { {} },
+);
+
+sub _clear_gettext_ref {
+    my $self = shift;
+
+    %{ $self->_gettext_ref } = ();
+
+    return;
+}
+
 has _num_ref => (
     is       => 'ro',
     init_arg => undef,
@@ -55,17 +68,27 @@ sub _clear_num_ref {
     return;
 }
 
-has limit => (
-    is      => 'rw',
-    isa     => PositiveInt,
-    clearer => 'clear_limit',
+has error => (
+    is       => 'rw',
+    init_arg => undef,
+    writer   => '_error',
+    clearer  => '_clear_error',
 );
 
-has sleep_seconds => (
-    is  => 'ro',
-    isa => PositiveOrZeroInt,
+has translation_count => (
+    is       => 'rw',
+    init_arg => undef,
+    writer   => '_translation_count',
 );
 
+sub _translation_count_increment {
+    my $self = shift;
+
+    $self->_translation_count( $self->translation_count + 1 );
+
+    return;
+}
+
 has developer_language => (
     is      => 'ro',
     isa     => Str,
@@ -78,39 +101,51 @@ has language => (
     required => 1,
 );
 
-has automatic_comment => (
-    is  => 'rw',
-    isa => Str,
-);
-
-has debug_code => (
+has before_translation_code => (
     is  => 'ro',
     isa => CodeRef,
 );
 
-has error => (
-    is      => 'rw',
-    clearer => '_clear_error',
+has after_translation_code => (
+    is  => 'ro',
+    isa => CodeRef,
 );
 
+has comment => (
+    is      => 'rw',
+    isa     => Str,
+    clearer => '_clear_comment',
+);
+
+# Coding schema:
+# a .. w, z     => A .. W, Z
+# A .. W, Z     => YA .. YW, YZ
+# space         => YX
+# open, e.g. {  => XX
+# :             => XY
+# close, e.g. } => XZ
+# other         => XAA .. XPP
+#                  like hex but
+#                  0123456789ABCDEF is
+#                  ABCDEFGHIJKLMNOP
+# not valid     => XQ .. XW, YY
+
 my $encode_az = sub {
     my $inner = shift;
+    local *__ANON__ = '$encode_az->'; ## no critic (InterpolationOfMetachars LocalVars)
 
     my $encode_inner = sub {
         my ( $lc, $uc, $space, $colon, $other ) = @_;
+        local *__ANON__ = '$encode_inner->'; ## no critic (InterpolationOfMetachars LocalVars)
 
-        if ( defined $lc ) {
-            return uc $lc;
-        }
-        if ( defined $uc ) {
-            return q{Y} . $uc;
-        }
-        if ( defined $space ) {
-            return 'YX';
-        }
-        if ( defined $colon ) {
-            return 'XY';
-        }
+        defined $lc
+            and return uc $lc;
+        defined $uc
+            and return q{Y} . $uc;
+        defined $space
+            and return 'YX';
+        defined $colon
+            and return 'XY';
 
         $other = ord $other;
         $other > 255 ## no critic (MagicNumbers)
@@ -144,7 +179,7 @@ sub _encode_named_placeholder {
     ## no critic (EscapedMetacharacters)
     $placeholder =~ s{
         ( \\ \{ )
-        | \{ ( [^\}]* ) \}
+        | \{ ( [^\{\}]* ) \}
     }
     {
         $1
@@ -157,6 +192,7 @@ sub _encode_named_placeholder {
 
 my $decode_inner = sub {
     my $inner = shift;
+    local *__ANON__ = '$decode_inner->'; ## no critic (InterpolationOfMetachars LocalVars)
 
     my @chars = $inner =~ m{ (.) }xmsg;
     my $decoded = q{};
@@ -169,7 +205,7 @@ my $decode_inner = sub {
         }
         if ( $char eq q{Y} ) {
             @chars
-                or confess 'decode error Y';
+                or return "DECODE_ERROR_Y($inner)";
             my $char2 = shift @chars;
             $decoded .= $char2 eq q{X}
                 ? q{ }
@@ -178,26 +214,26 @@ my $decode_inner = sub {
         }
         if ( $char eq q{X} ) {
             @chars
-                or confess 'decode error Xn';
+                or return "DECODE_ERROR_Xn($inner)";
             my $char2 = shift @chars;
             if ( $char2 eq q{Y} ) {
                 $decoded .= q{:};
                 next CHAR;
             }
             @chars
-                or confess 'decode error Xnn';
+                or return "DECODE_ERROR_Xnn($inner)";
             my $char3 = shift @chars;
             my $decode_string = 'ABCDEFGHIJKLMNOP';
             my $index2 = index $decode_string, $char2;
             $index2 == -1 ## no critic (MagicNumbers)
-                and confess 'decode error X?';
+                and return "DECODE_ERROR_X?($inner)";
             my $index1 = index $decode_string, $char3;
             $index1 == -1 ## no critic (MagicNumbers)
-                and confess 'decode error Xn?';
+                and return "DECODE_ERROR_Xn?($inner)";
             $decoded .= chr $index2 * 16 + $index1; ## no critic (MagicNumbers)
             next CHAR;
         }
-        confess 'decode error';
+        return "DECODE_ERROR($inner)";
     }
 
     return $decoded;
@@ -218,6 +254,35 @@ sub _decode_named_placeholder {
     return $placeholder;
 }
 
+sub _prepare_plural {
+    my ( $self, $nplurals, $plural_code ) = @_;
+
+    exists $self->_plural_ref->{0}
+        and return;
+
+    ## no critic (MagicNumbers)
+    NUMBER:
+    for ( 0 .. 1000 ) {
+        my $plural = $plural_code->($_);
+        if ( $plural > ( $nplurals - 1 ) ) {
+            confess sprintf
+                'Using plural formula with value %s. Got index %s. nplurals is %s. Then the maximal expected index is %s',
+                $_,
+                $plural,
+                $nplurals,
+                $nplurals - 1;
+        }
+        if ( ! exists $self->_plural_ref->{$plural} ) {
+            $self->_plural_ref->{$plural} = $_;
+        }
+        $nplurals == ( keys %{ $self->_plural_ref } )
+            and last NUMBER;
+    }
+    ## use critic (MagicNumbers)
+
+    return;
+}
+
 sub translate { ## no critic (ExcessComplexity)
     my ( $self, $name_read, $name_write ) = @_;
 
@@ -244,70 +309,68 @@ sub translate { ## no critic (ExcessComplexity)
     $self->_clear_plural_ref;
     my $entry_ref = { encode_obj => $encode_obj };
 
-    MESSAGE:
-    for my $po ( @{$pos_ref}[ 1 .. $#{$pos_ref} ] ) {
-        $entry_ref->{msgid}
-            = $po->msgid
-            && $encode_obj->decode( $po->dequote( $po->msgid ) );
-        $entry_ref->{msgid_plural}
-            = defined $po->msgid_plural
-            && $encode_obj->decode( $po->dequote( $po->msgid_plural ) );
-        $entry_ref->{msgstr}
-            = defined $po->msgstr
-            && $po->dequote( $po->msgstr );
-        length $entry_ref->{msgstr}
-            and next MESSAGE;
-        $entry_ref->{msgstr_n} = {};
-        my $msgstr_n = $po->msgstr_n || {};
-        my $is_all_translated = 1;
-        for my $index ( 0 .. ( $nplurals - 1 ) ) {
-            $entry_ref->{msgstr_n}->{$index}
-                = defined $msgstr_n->{$index}
-                && $po->dequote( $msgstr_n->{$index} );
-            my $is_translated
-                = defined $entry_ref->{msgstr_n}->{$index}
-                && length $entry_ref->{msgstr_n}->{$index};
-            $is_all_translated &&= $is_translated;
-        }
-        $is_all_translated
-            and next MESSAGE;
-        if ( length $entry_ref->{msgid_plural} ) {
-            if ( $nplurals ) {
-                ## no critic (MagicNumbers)
-                NUMBER:
-                for ( 0 .. 1000 ) {
-                    my $plural = $plural_code->($_);
-                    if ( ! exists $self->_plural_ref->{$plural} ) {
-                        $self->_plural_ref->{$plural} = $_;
-                    }
-                    $nplurals == ( keys %{ $self->_plural_ref } )
-                        and last NUMBER;
-                }
-                ## use critic (MagicNumbers)
+    $self->_translation_count(0);
+    try {
+        MESSAGE:
+        for my $po ( @{$pos_ref}[ 1 .. $#{$pos_ref} ] ) { # without 0 = header
+            $self->_clear_comment;
+            $entry_ref->{msgid}
+                = $po->msgid
+                && $encode_obj->decode( $po->dequote( $po->msgid ) );
+            $entry_ref->{msgid_plural}
+                = defined $po->msgid_plural
+                && $encode_obj->decode( $po->dequote( $po->msgid_plural ) );
+            $entry_ref->{msgstr}
+                = defined $po->msgstr
+                && $po->dequote( $po->msgstr );
+            length $entry_ref->{msgstr}
+                and next MESSAGE;
+            $entry_ref->{msgstr_n} = {};
+            my $msgstr_n = $po->msgstr_n || {};
+            my $is_all_translated = 1;
+            for my $index ( 0 .. ( $nplurals - 1 ) ) {
+                $entry_ref->{msgstr_n}->{$index}
+                    = defined $msgstr_n->{$index}
+                    && $po->dequote( $msgstr_n->{$index} );
+                my $is_translated
+                    = defined $entry_ref->{msgstr_n}->{$index}
+                    && length $entry_ref->{msgstr_n}->{$index};
+                $is_all_translated &&= $is_translated;
             }
-            $self->_translate_named_plural($entry_ref, $po);
-            $self->_update_automatic_comment($po);
-            Locale::PO->save_file_fromarray($name_write, $pos_ref);
-            next MESSAGE;
+            $is_all_translated
+                and next MESSAGE;
+            if ( length $entry_ref->{msgid_plural} ) {
+                if ( $nplurals ) {
+                    $self->_prepare_plural($nplurals, $plural_code);
+                }
+                $self->_translate_named_plural($entry_ref, $po);
+            }
+            ## no critic (EscapedMetacharacters)
+            elsif ( $entry_ref->{msgid} =~ m{ \{ [^\{\}]+ \} }xms ) {
+                $self->_translate_named($entry_ref, $po);
+            }
+            ## use critic (EscapedMetacharacters)
+            elsif ( $entry_ref->{msgid} =~ m{ [%] (?: \d | [*] | quant ) }xms ) {
+                $self->_translate_gettext($entry_ref, $po);
+            }
+            else {
+                $self->_translate_simple($entry_ref, $po);
+            }
+            $self->_update_comment($po);
         }
-        if ( $entry_ref->{msgid} =~ m{ \{ [^\{\}]+ \} }xms ) { ## no critic (EscapedMetacharacters)
-            $self->_translate_named($entry_ref, $po);
-            $self->_update_automatic_comment($po);
+        if ( $self->translation_count ) {
             Locale::PO->save_file_fromarray($name_write, $pos_ref);
-            next MESSAGE;
         }
-        if ( $entry_ref->{msgid} =~ m{ [%] (?: \d | [*] | quant ) }xms ) {
-            $self->_translate_gettext($entry_ref, $po);
-            $self->_update_automatic_comment($po);
-            Locale::PO->save_file_fromarray($name_write, $pos_ref);
-            next MESSAGE;
-        }
-        $self->_translate_simple($entry_ref, $po);
-        $self->_update_automatic_comment($po);
-        Locale::PO->save_file_fromarray($name_write, $pos_ref);
     }
+    catch {
+        if ( $self->translation_count ) {
+            Locale::PO->save_file_fromarray($name_write, $pos_ref);
+        }
+        m{ \A \QAPI error\E | \A (?: Before | After ) \Q translation break\E \b }xms
+            or confess $_;
+    };
 
-    return;
+    return $self;
 }
 
 sub _encode_named {
@@ -317,6 +380,7 @@ sub _encode_named {
     $self->_clear_num_ref;
     my $encode_placeholder = sub {
         my ( $placeholder, $is_num ) = @_;
+        local *__ANON__ = '$encode_placeholder->'; ## no critic (InterpolationOfMetachars LocalVars)
         if ( $is_num ) {
             $self->_num_ref->{$num} = $placeholder;
             return $num++;
@@ -370,14 +434,14 @@ sub _translate_named_plural {
     my $msgid        = $entry_ref->{msgid};
     my $msgid_plural = $entry_ref->{msgid_plural};
     MSGSTR_N:
-    for my $number_of_plural_form ( sort keys %{ $self->_plural_ref } ) {
-        my $index = $number_of_plural_form - 1;
+    for my $index ( sort keys %{ $self->_plural_ref } ) {
         defined $entry_ref->{msgstr_n}->{$index}
             and length $entry_ref->{msgstr_n}->{$index}
             and next MSGSTR_N;
         my $any_msgid = $self->_encode_named(
-            $number_of_plural_form == 1 ? $msgid : $msgid_plural,
-            $number_of_plural_form,
+            $index
+                ? ( $msgid_plural, $self->_plural_ref->{$index} )
+                : $msgid,
         );
         my $any_msgstr = $self->_translate_with_api($any_msgid);
         $any_msgstr = $self->_decode_named($any_msgstr);
@@ -389,18 +453,17 @@ sub _translate_named_plural {
 }
 
 sub _encode_gettext_inner { ## no critic (ManyArgs)
-    my ( $self, $quant, $number, $singular, $plural, $zero ) = @_;
+    my ( $self, $quant, $number, $inner, $singular, $plural, $zero ) = @_;
 
-    $self->_plural_ref->{$number} = [
-        $quant,
+    $self->_gettext_ref->{$inner} ||= [
         map {
-            ( defined $_ && length $_ )
+            ( defined && length )
             ? $self->_translate_with_api($_)
             : undef;
         } $singular, $plural, $zero
     ];
 
-    return $encode_az->("*$number");
+    return $encode_az->("$quant,$number,$inner");
 }
 
 sub _encode_gettext {
@@ -410,12 +473,13 @@ sub _encode_gettext {
     $msgid =~ s{
         ( %% )                    # escaped
         |
-        [%] ( [*] | quant )       # quant
-        [(]
-            [%] ( \d+ )           # number
-            [,] ( [^,)]* )        # singular
+        [%] ( [*] | quant ) [(]   # quant
+        [%] ( \d+ ) [,]           # number
+        (                         # inner
+            ( [^,)]* )            # singular
             [,] ( [^,)]* )        # plural
             (?: [,] ( [^,)]* ) )? # zero
+        )
         [)]
         |
         [%] ( \d+ )               # simple
@@ -424,8 +488,8 @@ sub _encode_gettext {
         $1
         ? $1
         : $2
-        ? $self->_encode_gettext_inner($2, $3, $4, $5, $6)
-        : $encode_az->($7)
+        ? $self->_encode_gettext_inner($2, $3, $4, $5, $6, $7)
+        : $encode_az->($8)
     }xmsge;
     ## use critic (ComplexRegexes)
 
@@ -439,17 +503,17 @@ sub _decode_gettext_inner {
     if ( $inner =~ m{ \A ( \d+ ) \z }xms ) {
         return q{%} . $1;
     }
-    if ( $inner =~ m{ \A [*] ( \d+ ) \z }xms ) {
-        my $plural = $self->_plural_ref->{$1};
+    if ( $inner =~ m{ \A ( [*] | quant ) [,] ( \d+ ) [,] ( .* ) \z }xms ) {
+        my ( $quant, $number, $plural ) = ( $1, $2, $self->_gettext_ref->{$3} );
         return join q{},
             q{%},
-            $plural->[0],
+            $quant,
             q{(},
-            ( join q{,}, "%$1", grep { defined } @{$plural}[ 1 .. 3 ] ), ## no critic (MagicNumbers)
+            ( join q{,}, "%$number", grep { defined } @{$plural}[ 0 .. 2 ] ),
             q{)};
     }
 
-    confess "decode error gettext inner $inner";
+    return "DECODE_ERROR($inner)";
 }
 
 sub _decode_gettext {
@@ -470,7 +534,7 @@ sub _decode_gettext {
 sub _translate_gettext {
     my ( $self, $entry_ref, $po ) = @_;
 
-    $self->_clear_plural_ref;
+    $self->_clear_gettext_ref;
     my $msgid = $self->_encode_gettext( $entry_ref->{msgid} );
     my $msgstr = $self->_translate_with_api($msgid);
     $msgstr = $self->_decode_gettext($msgstr);
@@ -491,21 +555,23 @@ sub _translate_simple {
     return;
 }
 
-sub _update_automatic_comment {
+sub _update_comment {
     my ( $self, $po ) = @_;
 
-    defined $self->automatic_comment
+    defined $self->comment
         or return;
-    length $self->automatic_comment
+    length $self->comment
         or return;
-    my $automatic = $po->automatic;
-    if ( ! defined $automatic ) {
-        $po->automatic( $self->automatic_comment );
+    if ( ! defined $po->comment ) {
+        $po->comment( $self->comment );
         return;
     }
-    my @lines = $automatic =~ m{ [\n]* ( [^\n]+ ) }xmsg;
-    push @lines, $self->automatic_comment;
-    $po->automatic( join "\n", sort +uniq( @lines ) );
+    my @lines
+        = grep {
+            $_ ne $self->comment;
+        }
+        $po->comment =~ m{ [\n]* ( [^\n]+ ) }xmsg;
+    $po->comment( join "\n", $self->comment, @lines );
 
     return;
 }
@@ -513,26 +579,26 @@ sub _update_automatic_comment {
 sub _translate_with_api {
     my ( $self, $msgid ) = @_;
 
-    if ( $self->error || defined $self->limit && ! $self->limit ) {
-        return q{};
+    $self->error
+        and die "API error\n";
+    if ( $self->before_translation_code ) {
+        $self->before_translation_code->($self, $msgid)
+            or die "Before translation break\n";
     }
-    $self->limit
-        and $self->limit( $self->limit - 1 );
-    $self->debug_code
-        and $self->debug_code->( $self->developer_language, $msgid );
     my $msgstr = try {
         $self->translate_text($msgid);
     }
     catch {
-        $self->error( ( defined $_ && length $_ ) ? $_ : 'unknown error' );
+        $self->_error( ( defined && length ) ? $_ : 'unknown error' );
         q{};
     };
     $self->error
-        and return;
-    $self->debug_code
-        and $self->debug_code->( $self->language, $msgstr);
-    if ( $self->sleep_seconds && defined $msgstr && length $msgstr ) {
-        sleep $self->sleep_seconds;
+        and die "API error\n";
+    $msgstr =~ s{ [\x{0}\x{4}] }{}xmsg; # because of mo file conflicts
+    $self->_translation_count_increment;
+    if ( $self->after_translation_code ) {
+        $self->after_translation_code->($self, $msgid, $msgstr)
+            or die "After translation break\n";
     }
 
     return $msgstr;
@@ -560,7 +626,7 @@ $HeadURL: $
 
 =head1 VERSION
 
-0.003
+1.000
 
 =head1 SYNOPSIS
 
@@ -575,12 +641,10 @@ $HeadURL: $
     sub translate_text {
         my ( $self, $text ) = @_;
 
-        my $developer_language = $self->developer_language;
-        my $language           = $self->language;
         my $translation = MyTranslatorApi
             ->new(
-                from => $developer_language,
-                to   => $language,
+                from => $self->developer_language,
+                to   => $self->language,
             )
             ->translate($text);
 
@@ -590,17 +654,30 @@ $HeadURL: $
 How to use see L<Locale::Utils::Autotranslator::ApiMymemoryTranslatedNet|Locale::Utils::Autotranslator::ApiMymemoryTranslatedNet>.
 
     my $obj = MyAutotranslator->new(
-        language => 'de',
-        # optional
-        debug_code => sub {
-            my ( $language, $text ) = @_;
+        language                => 'de',
+        # all following parameters are optional
+        developer_language      => 'en', # en is the default
+        before_translation_code => sub {
+            my ( $self, $msgid ) = @_;
             ...
+            return 1; # true: translate, false: skip translation
+        },
+        after_translation_code  => sub {
+            my ( $self, $msgid, $msgstr ) = @_;
+            ...
+            return 1; # true: translate, false: skip translation
         },
     );
-    $obj->translate(
+    $identical_obj = $obj->translate(
         'mydir/de.pot',
         'mydir/de.po',
     );
+    my $translation_count = $obj->translation_count;
+
+Return code of E.g. you have a limit of 100 free translations or 10000 words for 1 day
+you can skip further translations by return any false.
+
+Use that methods for debugging output.
 
 =head1 DESCRIPTION
 
@@ -610,53 +687,27 @@ Base class to translate automaticly.
 
 =head2 method developer_language
 
-Set/get the language of all msgid's. The default is 'en';
+Get back the language of all msgid's. The default is 'en';
 
 =head2 method language
 
-Set/get the language you want to translate.
+Get back the language you want to translate.
 
-=head2 limit
+=head2 before_translation_code, after_translation_code
 
-E.g. you have a limit of 100 free translations in 1 day
-you can run that each day with that limit.
-After that method translate returns,
-maybe with left untranslated messages.
+Get back the code references:
 
-You can read back the left limit and use them on other files.
-It is also allowed to run method translate with limit 0 first.
-Then it will translate all missing stuff to q{}.
-
-=head2 method sleep_seconds
-
-To prevent an attack to the translation service
-you can sleep after each translation request.
-Set/get the seconds here.
-
-=head2 method debug_code
-
-Set/get a code reference to see what text is sent to the translation service
-and what you get back.
-
-See Synopsis for parameters.
-
-=head2 method error
-
-Get back the error message if method translate_text dies.
+$code_ref = $obj->before_translation_code;
+$code_ref = $obj->after_translation_code;
 
 =head2 method translate
 
-    $object->translate('dir/de.pot', 'dir/de.po');
+    $obj->translate('dir/de.pot', 'dir/de.po');
 
 That means:
 Read the de.pot file (also possible *.po).
-Translate the 1st missing stuff.
-Write back the de.po file.
-Look for more missing translations.
-Translate the next missing stuff.
-
-Why write back so often?
-If something is broken (die) during translation the error is set.
+Translate the missing stuff.
+Write back to de.po file.
 
 =head2 method translate_text
 
@@ -667,6 +718,27 @@ Check the code of
 L<Locale::Utils::Autotranslator::ApiMymemoryTranslatedNet|Locale::Utils::Autotranslator::ApiMymemoryTranslatedNet>
 to see how to implement.
 
+=head2 method comment
+
+Set a typical comment to mark the translation as translated by ... in API class.
+Get back that comment.
+
+E.g.
+
+    $self->comment('translated by: api.mymemory.translated.net');
+
+=head2 method translation_count
+
+Get back the count of translations.
+This is not the count of translated messages,
+this is the count of successful translate_text calls.
+
+my $translation_count = $obj->translation_count;
+
+=head2 method error
+
+Get back the error message if method translate_text dies.
+
 =head1 EXAMPLE
 
 Inside of this distribution is a directory named example.
@@ -724,7 +796,7 @@ Steffen Winkler
 
 =head1 LICENSE AND COPYRIGHT
 
-Copyright (c) 2014,
+Copyright (c) 2014 - 2015,
 Steffen Winkler
 C<< <steffenw at cpan.org> >>.
 All rights reserved.
