@@ -11,23 +11,29 @@ use MooX::StrictConstructor;
 use URI;
 use namespace::autoclean;
 
-our $VERSION = '0.001';
+our $VERSION = '1.000';
 
 extends qw(
     Locale::Utils::Autotranslator
 );
 
+has user_agent => (
+    is        => 'ro',
+    duck_type => 'request',
+    lazy      => 1,
+    default   => sub { LWP::UserAgent->new },
+);
+
 sub translate_text {
     my ( $self, $msgid ) = @_;
 
-    $self->automatic_comment('translated by: api.mymemory.translated.net');
+    $self->comment('translated by: api.mymemory.translated.net');
     my $uri = URI->new('http://api.mymemory.translated.net/get');
     $uri->query_form(
         q        => $msgid,
         langpair => join q{|}, $self->developer_language, $self->language,
     );
-    my $ua = LWP::UserAgent->new;
-    my $response = $ua->request(
+    my $response = $self->user_agent->request(
         GET
             $uri->as_string,
             'User-Agent'      => 'Mozilla/5.0 (Windows NT 5.1; rv:26.0) Gecko/20100101 Firefox/26.0',
@@ -65,24 +71,33 @@ $HeadURL: $
 
 =head1 VERSION
 
-0.001
+1.000
 
 =head1 SYNOPSIS
 
     use Locale::Utils::Autotranslator::ApiMymemoryTranslatedNet;
 
     my $obj = Locale::Utils::Autotranslator::ApiMymemoryTranslatedNet->new(
-        language => 'de',
-        # optional
-        debug_code => sub {
-            my ( $language, $text ) = @_;
+        language                => 'de',
+        # all following parameters are optional
+        developer_language      => 'en', # en is the default
+        before_translation_code => sub {
+            my ( $self, $msgid ) = @_;
             ...
+            return 1; # true: translate, false: skip translation
         },
+        after_translation_code  => sub {
+            my ( $self, $msgid, $msgstr ) = @_;
+            ...
+            return 1; # true: translate, false: skip translation
+        },
+        user_agent              => $my_user_agent,
     );
-    $obj->translate(
+    $identical_obj = $obj->translate(
         'mydir/de.pot',
         'mydir/de.po',
     );
+    my $translation_count = $obj->translation_count;
 
 =head1 DESCRIPTION
 
@@ -145,7 +160,7 @@ Steffen Winkler
 
 =head1 LICENSE AND COPYRIGHT
 
-Copyright (c) 2014,
+Copyright (c) 2014 - 2015,
 Steffen Winkler
 C<< <steffenw at cpan.org> >>.
 All rights reserved.
